@@ -76,10 +76,11 @@ from itertools import cycle, count, chain
 from argparse import ArgumentTypeError
 from contextlib import suppress
 from functools import update_wrapper
-from typing import Optional, Tuple, Union, Mapping, Any, List, Iterable
+from typing import Optional, Tuple, Union, Mapping, Any, List, Iterable, ByteString
 
-from ..lib.loader import resolve, EntryNotFound
 from ..lib.frame import Chunk
+from ..lib.tools import isbuffer
+from ..lib.loader import resolve, EntryNotFound
 
 
 class PythonExpression:
@@ -147,10 +148,10 @@ class PythonExpression:
             raise ArgumentTypeError(F'The provided expression could not be parsed: {definition!s}')
 
         if type(next(nodes)) != ast.Module:
-            raise ArgumentTypeError('unknown error parsing the expression')
+            raise ArgumentTypeError(F'unknown error parsing the expression: {definition!s}')
 
         if type(next(nodes)) != ast.Expr:
-            raise ArgumentTypeError('operation string is not a valid Python expression')
+            raise ArgumentTypeError(F'not a valid Python expression: {definition!s}')
 
         nodes = list(nodes)
         types = set(type(node) for node in nodes)
@@ -327,7 +328,7 @@ class DelayedArgumentDispatch:
         except EntryNotFound:
             return None
         else:
-            unit = unit and unit.assemble(*args)
+            unit = unit and unit.assemble(*args).detach()
             self.units[uhash] = unit
             return unit
 
@@ -341,12 +342,11 @@ class DelayedArgumentDispatch:
             handler = self.default if modifier is None else self.handlers[modifier]
             return handler(self.instance, data, *args)
         except KeyError:
-            import io
             unit = self._get_unit(modifier, *args)
             if not unit:
                 raise ArgumentTypeError(F'failed to build unit {modifier}')
-            with io.BytesIO(data) as stream:
-                return B''.join(stream | unit)
+            result = unit.process(data)
+            return result if isbuffer(result) else B''.join(result)
 
     def can_handle(self, modifier, *args):
         return modifier in self.handlers or bool(self._get_unit(modifier, *args))
@@ -436,14 +436,22 @@ class DelayedArgument(LazyEvaluation):
                 return name, arguments, expression[k + 1:]
         return None, (), expression
 
-    def __call__(self, data: Optional[bytearray] = None) -> bytes:
+    def __call__(self, data: Optional[Chunk] = None) -> bytes:
         arg = self.seed
         mod = iter(self.modifiers)
         if not self.finalized:
             mod = chain(((None, ()),), mod)
         for name, arguments in mod:
+            if isbuffer(arg):
+                arg = Chunk(arg)
+                with suppress(AttributeError):
+                    arg.meta.update(data.meta)
             try:
                 arg = self.handler(arg, name, *arguments)
+            except VariableMissing as v:
+                if data is not None:
+                    raise
+                raise TooLazy from v
             except Exception as E:
                 raise ArgumentTypeError(F'failed to apply modifier {name} to incoming data: {E}') from E
             if callable(arg):
@@ -635,13 +643,18 @@ class DelayedNumbinArgument(DelayedArgument):
         except Exception:
             return self._mbin(expression)
 
-    @handler.register('ev', final=True)
-    def ev(self, expression: str) -> Iterable[int]:
+    @handler.register('ev')
+    def ev(self, expression: Union[str, ByteString]) -> Iterable[int]:
         """
         Final modifier `ev:expression`; uses a `refinery.lib.argformats.PythonExpression`
         parser to process expressions that may contain the variable `N` whose value will be
         the size of the input data.
         """
+        if not isinstance(expression, str):
+            try:
+                expression = expression.decode('ascii')
+            except AttributeError:
+                return expression
         ev = self._EV_PARSER(expression)
         try:
             return self._iter(ev())
@@ -724,9 +737,9 @@ class DelayedRegexpArgument(DelayedArgument):
             from .patterns import formats, indicators
 
             def replace(match):
-                name = match.group(1)
+                name = match[1]
                 return '(?:{})'.format(formats.get(
-                    name, indicators.get(name, match.group(0))))
+                    name, indicators.get(name, match[0])))
 
             expression = re.sub(
                 R'\(\?\?({}|{})\)'.format(
@@ -755,7 +768,7 @@ class DelayedRegexpArgument(DelayedArgument):
         import re
 
         def y2r(match):
-            expr = match.group(0)
+            expr = match[0]
             if expr == B'??':
                 return B'.'
             if B'?' not in expr:
