@@ -18,5 +18,5 @@ class deob_ps1_literals(Deobfuscator):
     def deobfuscate(self, data):
         strlit = Ps1StringLiterals(data)
         @strlit.outside
-        def strip(m): return F'${m.group(1)}'
+        def strip(m): return F'${m[1]}'
         return self._SENTINEL.sub(strip, data)
